==> 1.19.4.zip: snowygui/common/src/main/kotlin/fr/username404/snowygui/gui/elements/ClickBox.kt <==
package fr.username404.snowygui.gui.elements

import com.mojang.blaze3d.vertex.DefaultVertexFormat
import com.mojang.blaze3d.vertex.PoseStack
import fr.username404.snowygui.Snowy.Companion.MissingComponent
import fr.username404.snowygui.config.Configuration
import com.mojang.blaze3d.vertex.VertexFormat
import fr.username404.snowygui.gui.ColoredElement
import fr.username404.snowygui.gui.feature.ButtonImpl
import fr.username404.snowygui.gui.feature.Category
import fr.username404.snowygui.gui.feature.Colors
import fr.username404.snowygui.utils.RenderingUtil.buffer
import fr.username404.snowygui.utils.RenderingUtil.colorShader
import fr.username404.snowygui.utils.RenderingUtil.endDraw
import fr.username404.snowygui.utils.RenderingUtil.prepareDraw
import fr.username404.snowygui.utils.RenderingUtil.tessellator
import io.github.config4k.extract
import net.minecraft.client.Minecraft
import net.minecraft.network.chat.Component
import org.jetbrains.annotations.ApiStatus
import java.util.TreeSet
import kotlin.collections.LinkedHashSet

@ApiStatus.Internal
class ClickBox(
    x: Double, y: Double,
    val name: Component = MissingComponent,
    override var color: Int = savedColors?.get(name.string) ?: Colors.BLUE.hexValue,
): ColoredElement(x, y, 80, 10, color, 0.5F) {
    fun isCategory(c: Category): Boolean = (name.string == c.box.name.string)
    val buttons: MutableSet<ButtonImpl> =
        if (sortAlphabetically)
            TreeSet(compareBy(String.CASE_INSENSITIVE_ORDER) { it.title })
        else LinkedHashSet()
    override fun display(stack: PoseStack?) {
        hidden = buttons.isEmpty() || hidden
        super.display(stack)
    }

    @JvmField
    val buttonsProgressBar: ColoredElement = object: ColoredElement(
        (x + width), y + height + 3,
        width = 3, height = clickboxHeightOffset - 6,
        opacity = 0.75F
    ) {
        override val color: Int get() = this@ClickBox.color
        override fun render(poseStack: PoseStack?) {
            prepareDraw()
            colorShader(); defaultRectFunc()
            endDraw()
        }
        init { height = 8 }
    }
    var barStage: Int = 1; private set
    fun scroll(d: Double, e: Double, supplied: Double) {
        if (isWithinBounds(d, e, offsetHeight = clickboxHeightOffset.toDouble())) {
            with(buttonsProgressBar) {
                if ((height > 8 || (supplied < 0)) && (((height < (buttons.size * 8)) && (height < originalHeight)) || (supplied > 0))) {
                    height -= supplied.toInt()
                    if (buttons.isNotEmpty()) (height / 8).let {
                        if (it > 0) barStage = it
                    }
                }
            }
        }
    }

    companion object {
        private val savedColors = "box_colors".let {
            with(Configuration()) {
                if (!hasPath(it)) null else extract<Map<String, Int>>(it)
            }
        }
        var sortAlphabetically: Boolean by Configuration
        const val buttonsMax: Short = 16 // TODO Remove the buttons limit
        const val clickboxHeightOffset: Int = 80
        private const val inclination: Double = 2.5
    }
    override fun render(poseStack: PoseStack?) {
        val currentHeight = y + (height + clickboxHeightOffset)
        prepareDraw()
        with(buffer) {
            begin(VertexFormat.Mode.TRIANGLE_FAN, DefaultVertexFormat.POSITION_COLOR)
            // Render the header:
            vertex(x, y + height, 0.0).colorEnd()
            vertex(x + width + inclination, y + height, 0.0).colorEnd()
            vertex(x + width, y, 0.0).colorEnd()
            vertex(x + inclination, y, 0.0).colorEnd()

            // Render the box:
            vertex(x, currentHeight, 0.0).colorEnd()
            vertex(x + width + inclination, currentHeight, 0.0).colorEnd()
            vertex(x + width + inclination, y + height, 0.0).colorEnd()
            tessellator.end()

            colorShader()
            begin(VertexFormat.Mode.DEBUG_LINES, DefaultVertexFormat.POSITION_COLOR)
            vertex(x + inclination, y + height, 0.0).colorEnd(Colors.WHITE_LINES.hexValue)
            vertex(x + width, y + height, 0.0).colorEnd(Colors.WHITE_LINES.hexValue)
            tessellator.end()
        }
        endDraw()

        if (buttons.isNotEmpty()) {
            buttonsProgressBar.apply {
                x = this@ClickBox.x + this@ClickBox.width - 3
                y = this@ClickBox.y + this@ClickBox.height + 3
            }.display(poseStack)
            buttons.forEachIndexed { num, button ->
                val fullHeight = (y + height.toDouble())..(y + height + clickboxHeightOffset)
                button.also {
                    it.x = x + 3
                    it.y = y + 3 + height + (((num + 1) - barStage) * 9)
                    it.hidden = if ((num + 1) <= 8) ((it.y) !in fullHeight) else ((it.y + it.height) !in fullHeight)
                }.display(poseStack)
            }
        }
        if (poseStack != null) {
            Minecraft.getInstance().font.draw(poseStack, name.string, x.toFloat() + 5, y.toFloat() + 1.5F, Colors.TRANSPARENT.hexValue)
        }
    }
}

==> 1.20.zip: snowygui/common/src/main/kotlin/fr/username404/snowygui/gui/elements/ClickBox.kt <==
package fr.username404.snowygui.gui.elements

import com.mojang.blaze3d.vertex.DefaultVertexFormat
import fr.username404.snowygui.Snowy.Companion.MissingComponent
import fr.username404.snowygui.config.Configuration
import com.mojang.blaze3d.vertex.VertexFormat
import fr.username404.snowygui.gui.ColoredElement
import fr.username404.snowygui.gui.feature.ButtonImpl
import fr.username404.snowygui.gui.feature.Category
import fr.username404.snowygui.gui.feature.Colors
import fr.username404.snowygui.utils.RenderingUtil.buffer
import fr.username404.snowygui.utils.RenderingUtil.colorShader
import fr.username404.snowygui.utils.RenderingUtil.endDraw
import fr.username404.snowygui.utils.RenderingUtil.prepareDraw
import fr.username404.snowygui.utils.RenderingUtil.tessellator
import io.github.config4k.extract
import net.minecraft.client.Minecraft
import net.minecraft.client.gui.Font
import net.minecraft.client.gui.GuiGraphics
import net.minecraft.network.chat.Component
import org.jetbrains.annotations.ApiStatus
import java.util.TreeSet
import kotlin.collections.LinkedHashSet

@ApiStatus.Internal
class ClickBox(
    x: Double, y: Double,
    val name: Component = MissingComponent,
    override var color: Int = savedColors?.get(name.string) ?: Colors.BLUE.hexValue,
): ColoredElement(x, y, 80, 10, color, 0.5F) {
    fun isCategory(c: Category): Boolean = (name.string == c.box.name.string)
    val buttons: MutableSet<ButtonImpl> =
        if (sortAlphabetically)
            TreeSet(compareBy(String.CASE_INSENSITIVE_ORDER) { it.title })
        else LinkedHashSet()
    override fun display(guiGraphics: GuiGraphics?) {
        hidden = buttons.isEmpty() || hidden
        super.display(guiGraphics)
    }

    @JvmField
    val buttonsProgressBar: ColoredElement = object: ColoredElement(
        (x + width), y + height + 3,
        width = 3, height = clickboxHeightOffset - 6,
        opacity = 0.75F
    ) {
        override val color: Int get() = this@ClickBox.color
        override fun render(guiGraphics: GuiGraphics?) {
            prepareDraw()
            colorShader(); defaultRectFunc()
            endDraw()
        }
        init { height = 8 }
    }
    var barStage: Int = 1; private set
    fun scroll(d: Double, e: Double, supplied: Double) {
        if (isWithinBounds(d, e, offsetHeight = clickboxHeightOffset.toDouble())) {
            with(buttonsProgressBar) {
                if ((height > 8 || (supplied < 0)) && (((height < (buttons.size * 8)) && (height < originalHeight)) || (supplied > 0))) {
                    height -= supplied.toInt()
                    if (buttons.isNotEmpty()) (height / 8).let {
                        if (it > 0) barStage = it
                    }
                }
            }
        }
    }

    companion object {
        private val savedColors = "box_colors".let {
            with(Configuration()) {
                if (!hasPath(it)) null else extract<Map<String, Int>>(it)
            }
        }
        var sortAlphabetically: Boolean by Configuration
        const val buttonsMax: Short = 16 // TODO Remove the buttons limit
        const val clickboxHeightOffset: Int = 80
        private const val inclination: Double = 2.5
    }
    override fun render(guiGraphics: GuiGraphics?) {
        val currentHeight = y + (height + clickboxHeightOffset)
        prepareDraw()
        with(buffer) {
            begin(VertexFormat.Mode.TRIANGLE_FAN, DefaultVertexFormat.POSITION_COLOR)
            // Render the header:
            vertex(x, y + height, 0.0).colorEnd()
            vertex(x + width + inclination, y + height, 0.0).colorEnd()
            vertex(x + width, y, 0.0).colorEnd()
            vertex(x + inclination, y, 0.0).colorEnd()

            // Render the box:
            vertex(x, currentHeight, 0.0).colorEnd()
            vertex(x + width + inclination, currentHeight, 0.0).colorEnd()
            vertex(x + width + inclination, y + height, 0.0).colorEnd()
            tessellator.end()

            colorShader()
            begin(VertexFormat.Mode.DEBUG_LINES, DefaultVertexFormat.POSITION_COLOR)
            vertex(x + inclination, y + height, 0.0).colorEnd(Colors.WHITE_LINES.hexValue)
            vertex(x + width, y + height, 0.0).colorEnd(Colors.WHITE_LINES.hexValue)
            tessellator.end()
        }
        endDraw()

        if (buttons.isNotEmpty()) {
            buttonsProgressBar.apply {
                x = this@ClickBox.x + this@ClickBox.width - 3
                y = this@ClickBox.y + this@ClickBox.height + 3
            }.display(guiGraphics)
            buttons.forEachIndexed { num, button ->
                val fullHeight = (y + height.toDouble())..(y + height + clickboxHeightOffset)
                button.also {
                    it.x = x + 3
                    it.y = y + 3 + height + (((num + 1) - barStage) * 9)
                    it.hidden = if ((num + 1) <= 8) ((it.y) !in fullHeight) else ((it.y + it.height) !in fullHeight)
                }.display(guiGraphics)
            }
        }
        guiGraphics?.run {
            with(Minecraft.getInstance().font) {
                drawInBatch(name.string,
                    (x + 5).toFloat(), (y + 2).toFloat(), Colors.TRANSPARENT.hexValue, false,
                    pose().last().pose(), bufferSource(), Font.DisplayMode.NORMAL, 0, 15728880, isBidirectional
                )
            }
        }
    }
}
